# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Authors: Guido Amoruso <guidonte@fluendo.com>
#          Olivier Tilloy <olivier@fluendo.com>

from elisa.core.utils import defer, locale_helper
from elisa.core.media_uri import MediaUri
from elisa.core.default_config import PICTURES_CACHE

from elisa.plugins.poblesec.base.list import BaseListController, \
                                             GenericListViewMode
from elisa.plugins.poblesec.base.preview_list import \
    MenuItemPreviewListController
from elisa.plugins.poblesec.base.coverflow import \
    ImageWithReflectionCoverflowController
from elisa.plugins.poblesec.base.grid import GridItemGridController
from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController
from elisa.plugins.poblesec.actions import OpenControllerAction

from elisa.plugins.rss.models import RssItemModel, RssFeedModel

from twisted.internet import task
from twisted.web.client import getPage

import os

try:
    from hashlib import md5
except ImportError:
    # hashlib is new in Python 2.5
    from md5 import md5


# TODO: factorize this code in the core of Elisa in a caching utilities module
def _thumbnail_file(thumbnail_uri):
    if not os.path.exists(PICTURES_CACHE):
        os.makedirs(PICTURES_CACHE, 0755)
    thumbnail = str(md5(str(thumbnail_uri)).hexdigest()) + '.jpg'
    # make sure the thumbnail path is a valid unicode string
    path = os.path.join(PICTURES_CACHE,
                        unicode(thumbnail, locale_helper.system_encoding()))
    return path


class RssReaderController(BaseListController):

    def initialize(self, feed=None, feed_list=[], rss_feed_class=RssFeedModel,
                   media_filter=None):
        """
        @param feed:           the feed to display
        @type feed:            L{elisa.plugins.rss.models.RssFeedModel}
        @param feed_list:      a list of feed information in tuple of
                               (name, uri, icon, [etc. whatever needed])
        @type feed_list:       tuple of C{str}
        @param rss_feed_class: the class of the RSS feed
        """
        self.feed = feed
        self.feed_list = feed_list
        self.rss_feed_class = rss_feed_class
        self.media_filter = media_filter
        return super(RssReaderController, self).initialize()

    def populate_model(self):
        if self.feed is not None:
            def filter_feed_items(model):
                for item in self.feed.items:
                    if item.type.startswith(self.media_filter):
                        model.append(item)
                    yield None

            def feed_loaded(result):
                if self.media_filter is not None:
                    model = []
                    dfr = task.coiterate(filter_feed_items(model))
                    dfr.addCallback(lambda result: model)
                    return dfr
                else:
                    return self.feed.items

            dfr = self.feed.load()
            dfr.addCallback(feed_loaded)
            return dfr

        else:
            def iterate_feeds(model):
                for feed in self.feed_list:
                    feed_model = self.rss_feed_class()
                    feed_model.name = feed[0]
                    feed_model.uri = MediaUri(feed[1])
                    model.append(feed_model)
                    yield None

            model = []
            dfr = task.coiterate(iterate_feeds(model))
            dfr.addCallback(lambda result: model)
            return dfr

    def item_activated(self, item):
        if isinstance(item, RssFeedModel):
            action = OpenControllerAction(self, None)
            return action.open_controller(self.path, item.name, feed=item,
                                          rss_feed_class=self.rss_feed_class,
                                          media_filter=self.media_filter)
        elif isinstance(item, RssItemModel):
            poblesec = self.frontend.retrieve_controllers('/poblesec')[0]

            if item.video_model is not None:
                player = poblesec.video_player.player
                poblesec.show_video_player()
                player.play_model(item.video_model)
                return defer.succeed(None)

            elif item.track_model is not None:
                player = poblesec.music_player.player
                poblesec.show_music_player()
                player.play_model(item.track_model)
                return defer.succeed(None)

            elif item.image_model is not None:
                def iterate_images(playlist):
                    for image in self.model:
                        image_model = image.image_model
                        if image_model is not None:
                            playlist.append(image_model)
                        yield None

                def enqueue_and_play(playlist):
                    player = poblesec.slideshow_player.player
                    player.set_playlist(playlist)
                    poblesec.show_slideshow_player()
                    player.start_slideshow(playlist.index(item.image_model))

                playlist = []
                dfr = task.coiterate(iterate_images(playlist))
                dfr.addCallback(lambda result: playlist)
                dfr.addCallback(enqueue_and_play)
                return dfr

        # This should not happen if the feed was correctly formed and parsed.
        msg = 'The item does not contain a valid media model.'
        return defer.fail(TypeError(msg))


class RssReaderViewMode(GenericListViewMode):

    def get_label(self, item):
        if isinstance(item, RssFeedModel):
            return defer.succeed(item.name)
        elif isinstance(item, RssItemModel):
            return defer.succeed(item.title)

    def get_default_image(self, item):
        if isinstance(item, RssFeedModel):
            resource = 'elisa.plugins.poblesec.glyphs.small.devices_shares'
        elif isinstance(item, RssItemModel):
            if item.type.startswith('image'):
                resource = 'elisa.plugins.poblesec.glyphs.small.pictures'
            elif item.type.startswith('audio'):
                resource = 'elisa.plugins.poblesec.glyphs.small.music'
            elif item.type.startswith('video'):
                resource = 'elisa.plugins.poblesec.glyphs.small.video'
            else:
                resource = None
        return resource

    def get_image(self, item, theme):
        if isinstance(item, RssItemModel):
            # FIXME: copy-pasted implementation spread all over the code.
            if not(item.thumbnails):
                return None

            thumbnail_uri = str(item.thumbnails[0].references[0])
            if not getattr(item, 'thumbail_file', None):
                item.thumbnail_file = _thumbnail_file(thumbnail_uri)

            if os.path.exists(item.thumbnail_file):
                return defer.succeed(item.thumbnail_file)
            else:
                def got_thumbnail(response):
                    fd = file(item.thumbnail_file, 'wb')
                    fd.write(response)
                    fd.close()
                    return item.thumbnail_file

                dfr = getPage(thumbnail_uri)
                dfr.addCallback(got_thumbnail)
                return dfr
        else:
            return None

    def get_preview_image(self, item, theme):
        if isinstance(item, RssItemModel):
            try:
                return item.thumbnail_file
            except AttributeError:
                return None
        else:
            return None


class RssReaderPreviewListController(RssReaderController, MenuItemPreviewListController):
    view_mode = RssReaderViewMode


class RssReaderCoverflowController(RssReaderController, ImageWithReflectionCoverflowController):
    view_mode = RssReaderViewMode


class RssReaderGridController(RssReaderController, GridItemGridController):
    view_mode = RssReaderViewMode


class RssReaderListSwitcherController(ListSwitcherController):
    modes = [RssReaderPreviewListController,
             RssReaderCoverflowController,
             RssReaderGridController]
    default_config = {'view_mode': 'preview_list'}
